#!/usr/bin/env python

#
# Android vCard cleaner
# By PieroV <vogliadifarniente@gmail.com>
#
# This file is released under the public domain.
#

#
# Updated on Agoust, 27th, 2016 to drop vCards without names.
#

# Settings
# File exported from phone
FILE_IN = "contacts.vcf"
FILE_OUT = "contacts_clean.vcf"
# End of Settings

from vobject.base import readComponents

# Iterates the vCards of a stream
# Same parameters from vobject.base.readOne
def iterateContacts(stream, validate = False, transform = True, ignoreUnreadable = False, allowQP = False):
	return readComponents(stream, validate, transform, ignoreUnreadable, allowQP)

# Merge two vCards, adding missing info from b to a
def mergeVcards(a, b):
	for i in b.contents:
		if not i in a.contents:
			a.add(i)
			a.get(i).value = b.get(i).value

# Some contacts have double phone number, this function strips duplicates.
def removeDoubleTel(vcard):
	for key, value in vcard.contents.iteritems():
		if key != 'tel':
			continue

		i = 0
		while(i < len(value)):
			j = i + 1
			while(j < len(value)):
				if value[i].value == value[j].value:
					value.pop(j)
					j -= 1
				j += 1
			i += 1

		vcard.contents['tel'] = value

# Return a vcard name to sort by name
def orderByName(a, b):
	try:
		n1 = str(a.n.value)
	except:
		n1 = ''

	try:
		n2 = str(b.n.value)
	except:
		n2 = ''

	# Note that we want empty string at the end
	if a == b:
		return 0
	elif n1 == '' or n2 < n1:
		return 1
	else:
		return -1

# The main function
def main():
	fin = open(FILE_IN)
	vcards = {}
	for vcard in iterateContacts(fin):
		try:
			# In this way, if a vCard hasn't a name, it's automatically dropped
			vcard.n

			phone = vcard.tel.value
			removeDoubleTel(vcard)

			if phone in vcards:
				mergeVcards(vcards[phone], vcard)
			else:
				vcards[phone] = vcard

		except:
			pass

	# We want to sort by name contacts, so we don't care anymore of the phone
	# numbers as dictionary key, we need just the list, to sort it
	vcards = vcards.values()
	vcards.sort(orderByName)

	fout = open(FILE_OUT, 'w')
	for i in vcards:
		fout.write(i.serialize() + '\n')

if __name__ == '__main__':
	main()
